@@ -1,13 +1,15 @@
-;;; -*- Mode: Lisp ; Base: 10 ; Syntax: ANSI-Common-Lisp -*-
-;;; Scribble: SCRibe-like reader extension for Common Lisp
-;;; $Id: scribble.lisp,v 1.19 2003/09/10 15:04:21 fare Exp $
+#|" -*- Mode: Lisp ; Base: 10 ; Syntax: ANSI-Common-Lisp -*-
+Scribble: SCRibe-like reader extension for Common Lisp
+Copyright (c) 2002-2006 by Fare Rideau < fare at tunes dot org >
+	http://www.cliki.net/Fare%20Rideau
 
-#| "
 HOME PAGE:
 	http://www.cliki.net/Scribble
 
 LICENSE:
 	http://www.geocities.com/SoHo/Cafe/5947/bugroff.html
+You may at your leisure use the LLGPL instead:
+	http://www.cliki.net/LLGPL
 
 DEPENDENCY:
 This package depends on Meta by Jochen Schmidt, version 0.1.1 or later.
@@ -166,6 +168,7 @@ to accomodate the specificities of your markup language backend.
 
 * Example functions to customize scribble for use with various backends
  are given at the end of this file. Check functions
+	scribble:configure-scribble
 	scribble:configure-scribble-for-araneida
 	scribble:configure-scribble-for-htmlgen
 	scribble:configure-scribble-for-lml2
@@ -185,7 +188,7 @@ EXAMPLE USE:
 
 (let ((p "/home/fare/fare/www/liberty/white_black_magic.scr")
       (eof '#:eof))
-  (with-open-file (s p :direction :input :if-exists :error)
+  (with-open-file (s p :direction :input :if-does-not-exist :error)
     (loop for i = (read s nil eof nil)
       until (eq i eof)
       collect i)))
@@ -199,32 +202,8 @@ TODO:
 Share and enjoy!
 " |#
 
-; -----------------------------------------------------------------------------
-;;; Packaging stuff
-#-fare-scribble
-(eval-when (:compile-toplevel :load-toplevel :execute)
-  (asdf:oos 'asdf:load-op "meta")
-  (defpackage #:scribble
-    (:use #:meta #:common-lisp)
-    (:export #:enable-scribble-syntax #:disable-scribble-syntax
-	     #:reenable-scribble-syntax
-	     #:enable-sub-scribble-syntax #:disable-sub-scribble-syntax
-	     #:reenable-sub-scribble-syntax
-	     #:*scribble-preprocess* #:pp #:with-preprocessor
-	     #:*scribble-list*
-	     #:*scribble-default-head* #:default-scribble-list
-	     #:*scribble-package* #:*scribble-cons* #:default-scribble-cons
-	     #:configure-scribble-for-araneida
-	     #:configure-scribble-for-htmlgen
-	     #:configure-scribble-for-lml2
-	     #:configure-scribble-for-tml
-	     #:configure-scribble-for-who
-	     #:configure-scribble-for-yaclml))
-  (push :fare-scribble *features*))
-
 (in-package :scribble)
 
-
 ; -----------------------------------------------------------------------------
 ;;; Optimization
 (declaim (optimize (speed 3) (safety 1) (debug 0)))
@@ -248,10 +227,10 @@ in your macros so as to customize the behaviour of preprocessing"
 into preprocessing forms")
 
 (defun scribble-preprocess (s)
-  (cond
-   ((functionp *scribble-preprocess*) (funcall *scribble-preprocess* s))
-   (*scribble-preprocess* `(pp ,s))
-   (t s)))
+  (typecase *scribble-preprocess*
+    (null s)
+    ((or symbol function) (funcall *scribble-preprocess* s))
+    (t `(pp ,s))))
 
 ;-----------------------------------------------------------------------------
 ;;; Customizing components combination
@@ -287,7 +266,7 @@ the head of text in bracket-colon syntax")
 
 (defmacro within-package (package &body body)
   "do stuff while binding *package* to package if not NIL at runtime"
-  `(let ((package ,package) (fun (lambda () ,@body)))
+  `(let ((package ,package) (fun #'(lambda () ,@body)))
      (if package
 	 (let ((*package* (find-package package))) (funcall fun))
        (funcall fun))))
@@ -311,6 +290,14 @@ scribble returns from the head and body of text in bracket-colon syntax")
 (defun scribble-cons-with-list-head (head body)
    (cons (ensure-list head) body))
 
+; -----------------------------------------------------------------------------
+;;; Some error handling
+(defun issue-parse-error (fmt &rest r)
+    #+sbcl (error 'sb-int:simple-parse-error
+		  :format-control fmt
+		  :format-arguments r)
+    #-sbcl (apply 'error fmt r))
+
 ; -----------------------------------------------------------------------------
 ;;; The META parser
 
@@ -319,7 +306,7 @@ scribble returns from the head and body of text in bracket-colon syntax")
   '(member #\space #\newline #\tab #\linefeed #\return #\page))
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
-  (enable-meta-syntax))
+ (enable-meta-syntax))
 (defun parse-bracket (stream &aux c (s (make-string-output-stream)) (l '()))
   (with-stream-meta (st stream)
    (labels
@@ -346,15 +333,18 @@ scribble returns from the head and body of text in bracket-colon syntax")
        (match {[@(spacing-character c) !(skip-spaces)]}))
      (body ()
        (match
-        {[#\[ !(error "Nested bracket only allowed after backslash or comma.")]
+        {[#\[ !(issue-parse-error
+		"Nested bracket neither after backslash or comma on ~A @ ~A."
+		stream (file-position stream))]
 	 [#\] !(progn
 		(flush)
 		(close s)
-		(reverse l))]
+		(return-from body (reverse l)))]
 	 [#\, { [#\( !(progn (flush)
 			     (add (read-delimited-list #\) st t))
 			     (body))]
 	        [#\, !(progn (flush)
+			     (unread-char #\, st)
 			     (add (read-preserving-whitespace st t t t))
 			     (body))]
 	        [#\[ !(progn (flush)
@@ -389,7 +379,7 @@ scribble returns from the head and body of text in bracket-colon syntax")
   (set-macro-character #\]
       #'(lambda (stream char)
       (declare (ignore char))
-      (error 'reader-error :stream stream)))
+      (issue-parse-error "] outside of a [ construct on ~A @ ~A." stream (file-position stream))))
   (set-macro-character #\[
       #'(lambda (stream char)
       (declare (ignore char))
@@ -410,7 +400,7 @@ scribble returns from the head and body of text in bracket-colon syntax")
   (set-macro-character #\]
       #'(lambda (stream char)
       (declare (ignore char))
-      (error 'reader-error :stream stream)))
+      (issue-parse-error "] outside of a #[ construct on ~A @ ~A." stream (file-position stream))))
   (set-dispatch-macro-character #\# #\[
       #'(lambda (stream subchar arg)
 	  (declare (ignore subchar arg))
@@ -432,62 +422,63 @@ scribble returns from the head and body of text in bracket-colon syntax")
 ; Note that you must still independently (enable-scribble-syntax)
 ; or (enable-sub-scribble-syntax).
 
-(defun configure-scribble-for-araneida ()
-  "This will make Scribble work with my patched version of Araneida's html function that handles 'list correctly. Hopefully my patch will be integrated into CVS."
-  (setf *scribble-preprocess* nil
-	*scribble-list* 'default-scribble-list
-	*scribble-default-head* 'list
-	*scribble-package* nil
-	*scribble-cons* 'scribble-cons-with-list-head)
+(defun configure-scribble (&key (preprocess nil)
+				(list 'default-scribble-list)
+				(default-head 'list)
+				(package nil)
+				(cons 'default-scribble-cons))
+  (setf *scribble-preprocess* preprocess
+	*scribble-list* list
+	*scribble-default-head* default-head
+	*scribble-package* package
+	*scribble-cons* cons)
   t)
 
+#|
+(defun configure-scribble-for-exscribe ()
+  "This will make Scribble work with exscribe"
+  (configure-scribble :package :exscribe-user
+		      :cons 'default-scribble-cons
+		      :list 'default-scribble-list
+		      :default-head 'klist
+		      :preprocess nil))
+|#
+
+(defun configure-scribble-for-araneida ()
+  "This will make Scribble work with the patched version of araneida's original html.lisp function that I used in CTO and that handles 'list correctly. Hopefully my patch will be integrated into the main upstream darcs repository."
+  (configure-scribble :cons 'scribble-cons-with-list-head))
+
 (defun configure-scribble-for-htmlgen ()
   "This is meant to make Scribble work with AllegroServe's HTMLGEN from Franz, Inc. -- a least if I read the spec correctly."
-  (setf *scribble-preprocess* nil
-	*scribble-list* 'default-scribble-list
-	*scribble-default-head* (read-from-string "net.html.generator:html")
-	*scribble-package* (find-package '#:keyword)
-	*scribble-cons* 'cons)
-  t)
+  (configure-scribble :cons 'cons
+		      :default-head (read-from-string "net.html.generator:html")
+		      :package (find-package '#:keyword)))
 
 (defun configure-scribble-for-lml2 ()
   "This makes Scribble work with LML2 by kmr,
 which is based on Franz's HTMLGEN."
-  (setf *scribble-preprocess* nil
-	*scribble-list* 'default-scribble-list
-	*scribble-default-head* (read-from-string "lml2:html")
-	*scribble-package* (find-package '#:keyword)
-	*scribble-cons* 'cons)
-  t)
+  (configure-scribble :default-head (read-from-string "lml2:html")
+		      :package (find-package '#:keyword)
+		      :cons 'cons))
 
 (defun configure-scribble-for-tml ()
   "tml, previously known as htout, is tfeb's package.
 This is a wild guess from reading the docs.
 Please modify to actually suit the package."
-  (setf *scribble-preprocess* nil
-	*scribble-list* 'default-scribble-list
-	*scribble-default-head* (read-from-string "org.tfeb.tml:htm")
-	*scribble-package* (find-package '#:keyword)
-	*scribble-cons* 'cons)
-  t)
+  (configure-scribble :default-head (read-from-string "org.tfeb.tml:htm")
+		      :package (find-package '#:keyword)
+		      :cons 'cons))
 
 (defun configure-scribble-for-who ()
   "WHO is an optimized html generation package by Edi Weitz.
 Its keyword semantics is very Scribe-like.
 I wrote this reading the docs, but didn't test it."
-  (setf	*scribble-preprocess* nil
-	*scribble-list* 'default-scribble-list
-	*scribble-default-head* (read-from-string "who:htm")
-	*scribble-package* (find-package '#:keyword)
-	*scribble-cons* 'default-scribble-cons)
-  t)
+  (configure-scribble :default-head (read-from-string "who:htm")
+		      :package (find-package '#:keyword)))
 
 (defun configure-scribble-for-yaclml ()
   "yaclml is yet another common lisp markup language.
 The author wrote this support, but didn't test it."
-  (setf *scribble-preprocess* nil
-	*scribble-list* 'default-scribble-list
-	*scribble-default-head* (read-from-string "yaclml:yaclml-quote")
-	*scribble-package* (find-package '#:it.bese.yaclml)
-	*scribble-cons* 'cons)
-  t)
+  (configure-scribble :default-head (read-from-string "yaclml:yaclml-quote")
+		      :package (find-package '#:it.bese.yaclml)
+		      :cons 'cons))
